package xyz.zhouxy.plusone.system.domain.model.account;

import java.util.Objects;
import java.util.regex.Pattern;

import cn.hutool.core.util.DesensitizedUtil;
import xyz.zhouxy.plusone.commons.annotation.StaticFactoryMethod;
import xyz.zhouxy.plusone.commons.constant.PatternConsts;

/**
 * 值对象：手机号码
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
public class MobilePhone extends Principal {

    public static final Pattern REGEX = PatternConsts.MOBILE_PHONE;

    private MobilePhone(String mobilePhone) {
        super(REGEX);
        if (mobilePhone == null) {
            throw new IllegalArgumentException("手机号不能为空");
        }
        this.value = mobilePhone;
        if (!isValid()) {
            throw new IllegalArgumentException("手机号格式错误");
        }
    }

    @StaticFactoryMethod(MobilePhone.class)
    public static MobilePhone of(String mobilePhone) {
        return new MobilePhone(mobilePhone);
    }

    @StaticFactoryMethod(MobilePhone.class)
    public static MobilePhone ofNullable(String mobilePhone) {
        return Objects.nonNull(mobilePhone) ? new MobilePhone(mobilePhone) : null;
    }

    public String safeValue() {
        return DesensitizedUtil.mobilePhone(this.value);
    }

    @Override
    public String toString() {
        return this.safeValue();
    }
}
